/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"

%{C++
// {9E9B565A-8E52-4C1A-8805-B2B91655F874}
#define NS_ENTITYCONVERTER_CID { 0x9e9b565a, 0x8e52, 0x4c1a, {0x88, 0x05, 0xb2, 0xb9, 0x16, 0x55, 0xf8, 0x74}}
#define NS_ENTITYCONVERTER_CONTRACTID "@mozilla.org/intl/entityconverter;1"
%}

[scriptable, uuid(D14C7111-55E0-11d3-91D9-00105AA3F7DC)]
interface nsIEntityConverter : nsISupports
{
  const unsigned long entityNone = 0;
  const unsigned long html40Latin1 = 1;
  const unsigned long html40Symbols = 2;
  const unsigned long html40Special = 4;                       // excludes &quot, &amp, &lt, &gt
  const unsigned long transliterate = 8;
  const unsigned long mathml20 = 16;
  const unsigned long html32 = html40Latin1;
  const unsigned long html40 = html40Latin1+html40Symbols+html40Special;
  const unsigned long entityW3C = html40+mathml20;

  string ConvertUTF32ToEntity(in unsigned long character, in unsigned long entityVersion);
  string ConvertToEntity(in wchar character, in unsigned long entityVersion);

  wstring ConvertToEntities(in wstring inString, in unsigned long entityVersion);
};
