/* -*- Mode: IDL; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

#include "nsISupports.idl"
#include "nsIDOMWindow.idl"

[scriptable,uuid(ff9c0e45-4646-45ec-b2f0-3b16d9e41875)]
interface nsITabSource : nsISupports
{
  nsIDOMWindow getTabToStream();
  void notifyStreamStart(in nsIDOMWindow window);
  void notifyStreamStop(in nsIDOMWindow window);
};

%{C++
#define NS_TABSOURCESERVICE_CONTRACTID "@mozilla.org/tab-source-service;1"
%}
